/**
 * MEMOCODE Hardware/Software CoDesign Contest 2008
 *
 * This source file was derived from the public-domain 32-bit version of AES
 * by the authors as referenced below
 *
 * rijndael-alg-fst.c
 *
 * @version 3.0 (December 2000)
 *
 * Optimised ANSI C code for the Rijndael cipher (now AES)
 *
 * @author Vincent Rijmen <vincent.rijmen@esat.kuleuven.ac.be>
 * @author Antoon Bosselaers <antoon.bosselaers@esat.kuleuven.ac.be>
 * @author Paulo Barreto <paulo.barreto@terra.com.br>
 *
 * This code is hereby placed in the public domain.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef AES_CORE_H
#define AES_CORE_H

#define AES_MAXNR 10
#define AES_BLOCK_SIZE 16

struct aes_key_st {
    unsigned int rd_key[4 *(AES_MAXNR + 1)];
    int rounds;
};
typedef struct aes_key_st AES_KEY;

typedef unsigned int _u32_;
typedef unsigned char _u8_;

int AES_set_encrypt_key(const unsigned char *userKey, 
                        const int bits,
                        AES_KEY *key);

void AES_encrypt(const unsigned char *in, 
                 unsigned char *out,
                 const AES_KEY *key);

# define GETU32(pt) (((_u32_)(pt)[0] << 24) ^ ((_u32_)(pt)[1] << 16) ^ ((_u32_)(pt)[2] <<  8) ^ ((_u32_)(pt)[3]))
# define PUTU32(ct, st) { (ct)[0] = (_u8_)((st) >> 24); (ct)[1] = (_u8_)((st) >> 16); (ct)[2] = (_u8_)((st) >>  8); (ct)[3] = (_u8_)(st); }

#endif
